<?php

/**
 * SplClassLoader implementation that implements the technical interoperability
 * standards for PHP 5.3 namespaces and class names.
 *
 * http://groups.google.com/group/php-standards/web/final-proposal
 *
 *     // Example which loads classes for the Doctrine Common package in the
 *     // Doctrine\Common namespace.
 *     $classLoader = new SplClassLoader('Doctrine\Common', '/path/to/doctrine');
 *     $classLoader->register();
 *
 * @author Jonathan H. Wage <jonwage@gmail.com>
 * @author Roman S. Borschel <roman@code-factory.org>
 * @author Matthew Weier O'Phinney <matthew@zend.com>
 * @author Kris Wallsmith <kris.wallsmith@gmail.com>
 * @author Fabien Potencier <fabien.potencier@symfony-project.org>
*/

class NunoApps_Autoloader {

	private $_file_extension = '.php';
	private $_namespace;
	private $_include_path;
	private $_namespace_separator = '\\';

	/**
	 * Creates a new <tt>SplClassLoader</tt> that loads classes of the
	 * specified namespace.
	 * 
	 * @param string $ns The namespace to use.
	 */
	public function __construct( $ns = null, $include_path = null ) {
		$this->_namespace = $ns;
		$this->_include_path = $include_path;
	}

	/**
	 * Sets the namespace separator used by classes in the namespace of this class loader.
	 * 
	 * @param string $sep The separator to use.
	 */
	public function set_namespace_separator( $sep ) {
		$this->_namespace_separator = $sep;
	}

	/**
	 * Gets the namespace seperator used by classes in the namespace of this class loader.
	 *
	 * @return void
	 */
	public function get_namespace_separator() {
		return $this->_namespace_separator;
	}

	/**
	 * Sets the base include path for all class files in the namespace of this class loader.
	 * 
	 * @param string $include_path
	 */
	public function set_include_path( $include_path ) {
		$this->_include_path = $include_path;
	}

	/**
	 * Gets the base include path for all class files in the namespace of this class loader.
	 *
	 * @return string $include_path
	 */
	public function get_include_path() {
		return $this->_include_path;
	}

	/**
	 * Sets the file extension of class files in the namespace of this class loader.
	 * 
	 * @param string $file_extension
	 */
	public function set_file_extension( $file_extension ) {
		$this->_file_extension = $file_extension;
	}

	/**
	 * Gets the file extension of class files in the namespace of this class loader.
	 *
	 * @return string $file_extension
	 */
	public function get_file_extension() {
		return $this->_file_extension;
	}

	/**
	 * Installs this class loader on the SPL autoload stack.
	 */
	public function register() {
		spl_autoload_register( array( $this, 'load_class' ) );
	}

	/**
	 * Uninstalls this class loader from the SPL autoloader stack.
	 */
	public function unregister() {
		spl_autoload_unregister( array( $this, 'load_class' ) );
	}

	/**
	 * Loads the given class or interface.
	 *
	 * @param string $className The name of the class to load.
	 * @return void
	 */
	public function load_class( $class_name ) {

		if ( null === $this->_namespace || $this->_namespace . $this->_namespace_separator === substr( $class_name, 0, strlen( $this->_namespace . $this->_namespace_separator ) ) ) {
			$file_name = '';
			$namespace = '';

			if ( false !== ( $last_ns_pos = strripos( $class_name, $this->_namespace_separator ) ) ) {
				$namespace = substr( $class_name, 0, $last_ns_pos );
				$class_name = substr( $class_name, $last_ns_pos + 1 );
				$file_name = str_replace( $this->_namespace_separator, DIRECTORY_SEPARATOR, $namespace ) . DIRECTORY_SEPARATOR;
			}

			$file_name .= str_replace( '_', DIRECTORY_SEPARATOR, $class_name ) . $this->_file_extension;

			//wp_die(print_r($this->_include_path . DIRECTORY_SEPARATOR . $file_name, true));

			if ( file_exists( ( null !== $this->_include_path ? $this->_include_path . DIRECTORY_SEPARATOR : '' ) . $file_name ) ) {
				include_once( null !== $this->_include_path ? $this->_include_path . DIRECTORY_SEPARATOR : '' ) . $file_name;
			}
		}

	}
}